@@ -1,4 +1,4 @@
-.ablation.params.def <- read.table(header=TRUE, stringsAsFactors = FALSE, text="
+.ablation.params.def <- utils::read.table(header=TRUE, stringsAsFactors = FALSE, text="
 name            ab  type short long          default               description
 iraceResults     0  p    -l    --log-file    NA                    'Path to the (.Rdata) file created by irace from which the  \"iraceResults\" object will be loaded.'
 src              1  i    -S    --src         1                     'Source configuration ID or the path to a file containing the configuration.'
@@ -18,13 +18,15 @@ scenarioFile     0  p    -s    --scenario    NA                    'Scenario fil
 parallel         0  i    ''    --parallel    NA                    'Number of calls to targetRunner to execute in parallel. Values 0 or 1 mean no parallelization.'
 ")
 
+## __VERSION__ below will be replaced by the version defined in R/version.R
+## This avoids constant conflicts within this file.
 cat_ablation_license <- function()
 {
   ablation_license <-
 '#------------------------------------------------------------------------------
 # ablation: An implementation in R of Ablation Analysis
 # Version: __VERSION__
-# Copyright (C) 2020--2022
+# Copyright (C) 2020--2025
 # Manuel Lopez-Ibanez     <manuel.lopez-ibanez@manchester.ac.uk>
 # Leslie Perez Caceres    <leslie.perez.caceres@ulb.ac.be>
 #
@@ -40,11 +42,11 @@ cat_ablation_license <- function()
 #'
 #' Launch [ablation()] with the same command-line options as the command-line
 #' executable (`ablation.exe` in Windows).
-#' 
-#' @param argv `character()`\cr The arguments 
-#' provided on the R command line as a character vector, e.g., 
+#'
+#' @param argv `character()`\cr The arguments
+#' provided on the R command line as a character vector, e.g.,
 #' `c("-i", "irace.Rdata", "--src", 1)`.
-#' 
+#'
 #' @details The function reads the parameters given on the command line
 #' used to invoke R, launches [ablation()] and possibly [plotAblation()].
 #'
@@ -54,14 +56,15 @@ cat_ablation_license <- function()
 #' ```
 #' @return A list containing the following elements:
 #'  \describe{
-#'    \item{configurations}{Configurations tested in the ablation.}
-#'    \item{instances}{A matrix with the instances used in the experiments. First column has the 
-#'     instances IDs from \code{iraceResults$scenario$instances}, second column the seed assigned to the instance.}
+#'    \item{allConfigurations}{Configurations tested in the ablation.}
+#'    \item{state}{State of the ablation process.}
 #'    \item{experiments}{A matrix with the results of the experiments (columns are configurations, rows are instances).}
 #'    \item{scenario}{Scenario object with the settings used for the experiments.}
 #'    \item{trajectory}{IDs of the best configurations at each step of the ablation.}
 #'    \item{best}{Best configuration found in the experiments.}
+#'    \item{complete}{`TRUE` if the ablation process was completed.}
 #'  }
+#'
 #' @seealso [plotAblation()] [ablation()]
 #' @examples
 #' ablation_cmdline("--help")
@@ -84,7 +87,7 @@ ablation_cmdline <- function(argv = commandArgs(trailingOnly = TRUE))
   }
 
   if (!is.null(parser$readArg (short = "-v", long = "--version"))) {
-    print(citation(package="irace"))
+    print(utils::citation(package="irace"))
     return(invisible(NULL))
   }
 
@@ -95,12 +98,12 @@ ablation_cmdline <- function(argv = commandArgs(trailingOnly = TRUE))
     stop("Unknown command-line options: ", paste(parser$argv, collapse = " "))
 
   if (is.null(params$iraceResults)) {
-    irace.error("You must provide the path to the '.Rdata' file that contains the 'iraceResults' object generated by irace.")
+    irace_error("You must provide the path to the '.Rdata' file that contains the 'iraceResults' object generated by irace.")
     return(invisible(NULL))
   }
   iraceResults <- read_logfile(params$iraceResults)
   if (is.null(params$old_path) != is.null(params$new_path)) {
-    irace.error("To update paths you must provide both --old-path and --new-path.")
+    irace_error("To update paths you must provide both --old-path and --new-path.")
     return(invisible(NULL))
   } else if (!is.null(params$old_path)) {
     iraceResults$scenario <- scenario_update_paths(iraceResults$scenario, params$old_path, params$new_path)
@@ -113,21 +116,22 @@ ablation_cmdline <- function(argv = commandArgs(trailingOnly = TRUE))
     scenario$seed <- NULL
   }
   for (p in c("execDir", "parallel")) {
-    if (!is.null(params[[p]])) scenario[[p]] <- params[[p]]
+    if (!is.null(params[[p]]))
+      scenario[[p]] <- params[[p]]
   }
-  
+
   if (is_null_or_empty_or_na(trim(params$ablationLogFile))) {
     params$ablationLogFile <- NULL
   } else {
     params$ablationLogFile <- path_rel2abs(params$ablationLogFile)
   }
-  
+
   if (!is.null(params$ab_params))
     params$ab_params <- trimws(strsplit(params$ab_params, ",", fixed=TRUE)[[1L]])
 
   # The shell may introduce extra quotes, remove them.
   params$plot_type <- trimws(gsub("[\"']", "", params$plot_type))
-  
+
   # We want to select elements that actually appear in params, otherwise we get NA names.
   ablation_params <- intersect(.ablation.params.def[.ablation.params.def$ab == 1, "name", drop=TRUE],
                                names(params))
@@ -136,62 +140,106 @@ ablation_cmdline <- function(argv = commandArgs(trailingOnly = TRUE))
                             params[ablation_params],
                             scenario))
   if (!is.null(params[["plot"]]) || base::interactive()) {
-    plotAblation(ablog, pdf_file = params[["plot"]], type = params$plot_type) 
+    plotAblation(ablog, pdf_file = params[["plot"]], type = params$plot_type)
   }
   invisible(ablog)
 }
 
-## This function fixes dependent parameters when a parameter value has been
-## changed.
-fixDependenciesWithReference <- function(configuration, ref_configuration, parameters)
+fix_dependents <- function(parameters, dep_names, new_conf, final_conf, changed)
 {
-  # Search parameters that need a value
-  changed <- c()
-  for (pname in parameters$names_variable) {
-    # If dependent parameter has been activated, set the value of the reference.
-    if (is.na(configuration[[pname]]) && conditionsSatisfied(parameters$conditions[[pname]], configuration)) {
-       if (!is.null(ref_configuration)) {
-         configuration[[pname]] <- ref_configuration[[pname]]
-       } 
-       changed <- c(changed, pname)
-       # MANUEL: Why do we need to recurse here?
-       aux <- fixDependenciesWithReference(configuration=configuration, ref_configuration=ref_configuration, parameters)
-       changed <- c(changed, aux$changed)
-       configuration <- aux$configuration
+  # ... check every dependent parameter...
+  for (dep  in dep_names) {
+    # If the conditions are satisfied and it doesn't have a value but it
+    # should, then change it.  If the conditions are not satisfied and it
+    # has a value but it shouldn't, then change it.
+    ok <- conditionsSatisfied(parameters$conditions[[dep]], new_conf)
+    change <- ok == is.na(new_conf[[dep]])
+    dep_param <- parameters$get(dep)
+    if (change) {
+      # If it doesn't have a value but it should, take it from final.
+      if (ok) {
+        if (dep_param[["is_dependent"]]) {
+          # If the parameter is domain-dependent, then:
+          if (anyNA(new_conf[, .SD, .SDcols=all.vars(dep_param$domain)])) {
+            # If it depends on a parameter that is disabled, then this is disabled, so no need to change.
+            next
+          } else if (is.na(final_conf[[dep]]) # If the final value is NA, we cannot fix it, so skip it.
+            # If parameter X enables dep but dep has domain that depends on
+            # parameter Y, then Y in new_conf may have a value that makes
+            # final_conf[[dep]] invalid. We cannot fix this, so skip.
+            || !is_within_dependent_bound(dep_param, new_conf, value = final_conf[[dep]]))
+            return(NULL)
+        }
+        set(new_conf, j = dep, value = final_conf[[dep]])
+      } else {
+        set(new_conf, j = dep, value = NA)
+      }
+      changed <- c(changed, dep)
+    } else if (dep_param[["is_dependent"]] && !is.na(new_conf[[dep]])) {
+      # The condition may not have changed but the domain may depend on a parameter that has been changed.
+      if (anyNA(new_conf[, .SD, .SDcols=all.vars(dep_param$domain)])) {
+        set(new_conf, j = dep, value = NA)
+        changed <- c(changed, dep)
+      } else if (!is_within_dependent_bound(dep_param, new_conf, value = new_conf[[dep]])) {
+        # If the current value is not within bound and the final value is NA, we cannot fix it, so skip it.
+        if (is.na(final_conf[[dep]])
+          # If parameter X enables dep but dep has domain that depends on
+          # parameter Y, then Y in new_conf may have a value that makes
+          # final_conf[[dep]] invalid. We cannot fix this, so skip.
+          || !is_within_dependent_bound(dep_param, new_conf, value = final_conf[[dep]]))
+          return(NULL)
+        set(new_conf, j = dep, value = final_conf[[dep]])
+        changed <- c(changed, dep)
+      }
     }
   }
-  list(configuration=configuration, changed=changed)
+  changed
 }
 
-## Function that generates the configurations of the ablation path 
+## Function that generates the configurations of the ablation path
 ## between initial_configuration and final_configuration.
 ## parameters can be selected by specifying them in para.names.
-generateAblation <- function(initial_configuration, final_configuration,
-                             parameters, param_names = NULL)
-{ 
+generate_ablation <- function(initial_configuration, final_configuration,
+                              parameters, param_names)
+{
   # Only change variable parameters
-  if (is.null(param_names))
-    param_names <- parameters$names_variable
-  else 
-    param_names <- setdiff(param_names, parameters$names_fixed)
-  
-  configurations <- NULL
+  param_names <- setdiff(param_names, parameters$names_fixed)
+  # Follow the hierarchy order.
+  hierarchy <- sort(parameters$hierarchy)
+  param_names <- intersect(names(parameters$hierarchy), param_names)
+  configurations <- list()
   changed_params <- list()
+  # skip if the value is the same or NA (not active)
+  skip <- (initial_configuration[param_names] == final_configuration[param_names])
+  skip <- skip[1L, ] # Drop to a vector with names
+  skip <- skip | is.na(skip)
   for (pname in param_names) {
-    # Check if parameter is active.
-    if (!conditionsSatisfied(parameters$conditions[[pname]], initial_configuration)) next
-    # Check value is different in the initial and final configuration and if
-    # so, change the value.
-    if (initial_configuration[[pname]] == final_configuration[[pname]]) next
-    new_configuration <- initial_configuration
-    new_configuration[[pname]] <- final_configuration[[pname]]
-    # Set newly activated parameters if needed.
-    aux <- fixDependenciesWithReference(new_configuration, final_configuration, parameters)
-    new_configuration <- aux[["configuration"]] 
-    changed_params[[length(changed_params) + 1L]] <- c(pname, aux[["changed"]])
-    new_configuration[[".PARENT."]] <- initial_configuration[[".ID."]]
-    configurations <- rbind.data.frame(configurations, new_configuration)
+    # If parameter is not active, it will only change if its condition becomes true.
+    if (skip[[pname]]) next
+    param <- parameters$get(pname)
+    # If the parameter is domain-dependent, then we can only change its
+    # value if the new value is within the domain.
+    if (param[["is_dependent"]]
+      && !is_within_dependent_bound(param, initial_configuration, value = final_configuration[[pname]]))
+      next
+    new_configuration <- as.data.table(initial_configuration)
+    set(new_configuration, j = pname, value = final_configuration[[pname]])
+    changed <- pname
+    # If other parameters depend on this one then...
+    if (any(sapply(parameters$depends, function(x) any(pname %in% x)))) {
+      changed <- fix_dependents(parameters, dep_names = names(hierarchy)[hierarchy > hierarchy[[pname]]],
+        new_conf = new_configuration, final_conf = final_configuration, changed = changed)
+      if (is.null(changed))
+        next
+    }
+    new_configuration <- filter_forbidden(new_configuration, parameters$forbidden)
+    if (nrow(new_configuration) == 0L) next
+    changed_params <- c(changed_params, list(changed))
+    configurations <- c(configurations, list(new_configuration))
   }
+  configurations <- rbindlist(configurations)
+  set(configurations, j = ".PARENT.", value = configurations[[".ID."]])
+  setDF(configurations)
   rownames(configurations) <- NULL
   list(configurations=configurations, changed_params=changed_params)
 }
@@ -223,14 +271,14 @@ ab_generate_instances <- function(race_state, scenario, nrep, type, instancesFil
     stop("'nrep' has no effect when type == 'racing'")
   if (nrep > 1L && scenario$deterministic)
     stop("'nrep > 1' does not make sense with a deterministic scenario")
-  
+
   if (instancesFile == "test") {
     scenario$instances <- scenario$testInstances
   } else if (instancesFile != "train") {
     scenario$instances <- readInstances(instancesFile = path_rel2abs(instancesFile))
   }
   n_inst <- length(scenario$instances)
-  if (type == "full" && n_inst * nrep == 1)
+  if (type == "full" && n_inst * nrep == 1L)
     stop("'nrep' must be larger than 1 when type == 'full' and a single instance")
   generateInstances(race_state, scenario, n_inst * nrep)
 
@@ -263,7 +311,7 @@ ab_generate_instances <- function(race_state, scenario, nrep, type, instancesFil
 #' @references
 #' C. Fawcett and H. H. Hoos. Analysing differences between algorithm
 #' configurations through ablation. Journal of Heuristics, 22(4):431–458, 2016.
-#' 
+#'
 #' @inherit ablation_cmdline return
 #' @seealso [plotAblation()] [ablation_cmdline()]
 #' @examples
@@ -276,7 +324,7 @@ ab_generate_instances <- function(race_state, scenario, nrep, type, instancesFil
 #' # subset of parameters, directly reading the setup from the irace log file.
 #' ablog <- ablation(logfile, src = 1, target = 10,
 #'                   ab_params = c("temp"), ablationLogFile = NULL)
-#' plotAblation(ablog)
+#' plotAblation(ablog, type = "mean")
 #' }
 #'
 #' @author Leslie Pérez Cáceres and Manuel López-Ibáñez
@@ -289,33 +337,36 @@ ablation <- function(iraceResults, src = 1L, target = NULL,
                      instancesFile="train", ...)
 {
   # Input check
-  if (missing(iraceResults) || is.null(iraceResults)) 
+  if (missing(iraceResults) || is.null(iraceResults))
     stop("You must provide an 'iraceResults' object generated by irace or the path to the '.Rdata' file that contains this object.")
 
   type <- match.arg(type)
 
-  if (!is.null(ablationLogFile))
+  if (!is.null(ablationLogFile)) {
     file.check(ablationLogFile, writeable = TRUE, text = 'ablationLogFile')
-  
+  }
+
   save_ablog <- function(complete) {
     ablog <- list(changes = changes,
-                  configurations = all_configurations,
-                  experiments = results,
-                  instances   = race_state$instances_log,
-                  scenario    = scenario, 
-                  trajectory  = trajectory,
-                  best = best_configuration,
-                  complete = complete)
-    if (!is.null(ablationLogFile)) save(ablog, file = ablationLogFile, version = 3L)
-    ablog
+      # This name must match the one used in the irace log.
+      allConfigurations = as.data.frame(all_configurations),
+      experiments = experiments,
+      state = race_state,
+      scenario = scenario,
+      trajectory = trajectory,
+      best = best_configuration,
+      complete = complete)
+    if (!is.null(ablationLogFile))
+      save(ablog, file = ablationLogFile, version = 3L)
+    invisible(ablog)
   }
 
   # Load the data of the log file.
   iraceResults <- read_logfile(iraceResults)
   log_version <- get_log_clean_version(iraceResults)
   if (log_version < "3.9.0")
-    irace.error("The version of the logfile (", log_version, ") is too old for this version of ablation")
-  
+    irace_error("The version of the logfile (", log_version, ") is too old for this version of ablation")
+
   if (is.null(iraceResults$state$completed) || length(iraceResults$state$completed) != 1L
     || iraceResults$state$completed == "Incomplete")
     stop("The 'iraceResults' logfile seems to belong to an incomplete run of irace.")
@@ -344,7 +395,7 @@ ablation <- function(iraceResults, src = 1L, target = NULL,
     # FIXME: Check for duplicates
   } else if (src %not_in% iraceResults$allConfigurations[[".ID."]])
     stop("Source configuration ID (", src, ") cannot be found!")
- 
+
   if (is.null(target))
     target <- iraceResults$iterationElites[length(iraceResults$iterationElites)]
   else if (is.character(target) && is.na(suppressWarnings(as.integer(target)))) {
@@ -359,11 +410,11 @@ ablation <- function(iraceResults, src = 1L, target = NULL,
     # FIXME: Check for duplicates
   } else if (target %not_in% iraceResults$allConfigurations[[".ID."]])
     stop("Target configuration ID (", target, ") cannot be found!")
-  
+
   if (src == target)
     stop("Source and target configuration IDs must be different!")
-  
-  irace.note ("Starting ablation from ", src, " to ", target, "\n# Seed: ", race_state$seed, "\n")
+
+  irace_note ("Starting ablation from ", src, " to ", target, "\n# Seed: ", race_state$seed, "\n")
   cat("# Source configuration (row number is ID):\n")
   src_configuration <- iraceResults$allConfigurations[src, , drop = FALSE]
   configurations_print(src_configuration)
@@ -375,150 +426,112 @@ ablation <- function(iraceResults, src = 1L, target = NULL,
   if (is.null(ab_params)) {
     ab_params <- parameters$names
   } else if (!all(ab_params %in% parameters$names)) {
-    irace.error("Some of the parameters provided (", paste0(setdiff(ab_params, parameters$names), collapse=", "),
+    irace_error("Some of the parameters provided (", paste0(setdiff(ab_params, parameters$names), collapse=", "),
                 ") are not defined in the parameter space.")
   }
   # Select parameters that are different in both configurations
   neq_params <- which(src_configuration[,ab_params] != target_configuration[,ab_params])
-  if (length(neq_params) == 0L) 
-    irace.error("src and target configurations are equal considering the parameters selected.\n")
+  if (length(neq_params) == 0L)
+    irace_error("src and target configurations are equal considering the parameters selected.\n")
   param_names <- colnames(src_configuration[,ab_params])[neq_params]
-  
+
   # FIXME: Do we really need to override the ID?
-  src_configuration$.ID. <- best_id <-  1L
-  best_configuration <- all_configurations <- src_configuration
-  
-  # Execute source and target configurations.
-  ## FIXME: We may already have these experiments in the logFile!
-  experiments <- createExperimentList(configurations = rbind(src_configuration, target_configuration), 
-                                      parameters = parameters,
-                                      instances = scenario$instances,
-                                      instances_ID = race_state$instances_log[["instanceID"]],
-                                      seeds = race_state$instances_log[["seed"]],
-                                      bounds = scenario$boundMax)
-  irace.note("Executing source and target configurations on the given instances * nrep (", nrow(race_state$instances_log), ")...\n")
-  
+  src_configuration[[".ID."]] <- 1L
+  target_configuration[[".ID."]] <- 2L
+  all_configurations <- rbindlist(list(src_configuration, target_configuration), use.names=TRUE)
+
   race_state$start_parallel(scenario)
   on.exit(race_state$stop_parallel(), add = TRUE)
-  target_output <- execute_experiments(race_state, experiments, scenario)
-  if (!is.null(scenario$targetEvaluator))
-    target_output <- execute_evaluator (race_state$target_evaluator, experiments, scenario, target_output,
-                                        src_configuration)
-  # Save results
-  output <- sapply(target_output, getElement, "cost") 
-  results <- matrix(NA_real_, ncol = 1L, nrow = nrow(race_state$instances_log), 
-                    dimnames = list(seq_nrow(race_state$instances_log), 1L))
-  results[,1L] <- output[seq_nrow(race_state$instances_log)]
-  lastres <- output[(nrow(race_state$instances_log)+1L):(2L * nrow(race_state$instances_log))]
+
+  # Execute source and target configurations.
+  ## FIXME: We may already have these experiments in the logFile!
+  irace_note("Executing source and target configurations on the given instances * nrep (", nrow(race_state$instances_log), ")...\n")
+  experiments <- do_experiments(race_state,
+    configurations = rbind.data.frame(src_configuration, target_configuration),
+    ninstances = nrow(race_state$instances_log), scenario = scenario, iteration = 0L)
+  irace_assert(ncol(experiments) == 2L)
   step <- 1L
   # Define variables needed
   trajectory <- 1L
   names(trajectory) <- "source"
   # FIXME: changes should only store the changed parameters.
   changes <- list()
-  ablog <- save_ablog(complete = FALSE)
+  best_configuration <- src_configuration
+  save_ablog(complete = FALSE)
   while (length(param_names) > 1L) {
     # Generate ablation configurations
     cat("# Generating configurations (row number is ID):", param_names, "\n")
-    ab_aux <- generateAblation(best_configuration, target_configuration, parameters,
-                               param_names)
+    ab_aux <- generate_ablation(best_configuration, target_configuration, parameters,
+      param_names)
     aconfigurations <- ab_aux$configurations
-    if (is.null(aconfigurations)) {
+    if (nrow(aconfigurations) == 0L) {
       cat("# Stopping ablation, no parameter change possible.\n")
       break
     }
     ## FIXME: We may already have these configurations in the logFile!
-    # New configurations ids
-    ## FIXME: These should be generated with respect to the logFile to make
-    ## sure we don't have duplicate IDs.
-    aconfigurations[[".ID."]] <- max(0L, all_configurations[[".ID."]]) + seq_nrow(aconfigurations)
+    # New configurations IDs
+    aconfigurations[[".ID."]] <- max(all_configurations[[".ID."]]) + seq_nrow(aconfigurations)
     configurations_print(aconfigurations, metadata = FALSE)
-    all_configurations <- rbind(all_configurations, aconfigurations)
-    
-    # Set variables for the racing procedure
-    if (scenario$capping) {
-      # For using capping we must set elite data
-      elite_data <- list(experiments = results[,best_configuration[[".ID."]], drop=FALSE])
-      race_conf <-  rbind(best_configuration, aconfigurations)
-      race_state$next_instance <- nrow(race_state$instances_log) + 1L
-    } else {
-      #LESLIE: for now we apply the non-elitis irace when type=="racing"
-      # we should define what is the standard
-      elite_data <- NULL
-      race_conf <-  aconfigurations
-      scenario$elitist <- FALSE
-      race_state$next_instance <- 1L
-    }
-          
-    irace.note("Ablation (", type, ") of ", nrow(aconfigurations),
-               " configurations on ", nrow(race_state$instances_log), " instances.\n")
-    # Force the race to see all instances in "full" mode
-    # FIXME: Full mode should simply evaluate all instances in parallel.
-    if (type == "full") scenario$firstTest <- nrow(race_state$instances_log)
-    # FIXME: what about blockSize?
-    race_output <- elitist_race(race_state,
-      maxExp = nrow(aconfigurations) * nrow(race_state$instances_log),
-                                minSurvival = 1L,
-                                elite.data = elite_data,
-                                configurations = race_conf,
-                                scenario = scenario,
-                                elitist_new_instances = 0L)
-    results <- merge_matrix(results, race_output$experiments)
+    all_configurations <- rbindlist(list(all_configurations, aconfigurations), use.names=TRUE)
 
-    # Save log
-    ablog <- save_ablog(complete = FALSE)
-    
+    irace_note("Ablation (", type, ") of ", nrow(aconfigurations),
+      " configurations on ", nrow(race_state$instances_log), " instances (this may take a while ...).\n")
+    if (type == "full") {
+      step_experiments <- do_experiments(race_state,
+        configurations = aconfigurations,
+        ninstances = nrow(race_state$instances_log),
+        scenario = scenario, iteration = step)
+    } else {
+      # Set variables for the racing procedure
+      # FIXME: what about blockSize?
+      race_state$next_instance <- 1L
+      race_output <- elitist_race(race_state,
+        maxExp = nrow(aconfigurations) * nrow(race_state$instances_log),
+        minSurvival = 1L,
+        elite_data = NULL,
+        configurations = aconfigurations,
+        scenario = scenario,
+        elitist_new_instances = 0L)
+      set(race_output$experiment_log, j = "iteration", value = step)
+      race_state$experiment_log <-
+        rbindlist(list(race_state$experiment_log, race_output$experiment_log),
+          use.names=TRUE)
+      step_experiments <- race_output$experiments
+    }
+    experiments <- merge_matrix(experiments, step_experiments)
+
+    save_ablog(complete = FALSE)
     # Get the best configuration based on the criterion of irace
-    # MANUEL: Doesn't race_output already give you all this info???
-    cranks <- overall_ranks(results[,aconfigurations[[".ID."]],drop=FALSE], test = scenario$testType)
+    cranks <- overall_ranks(experiments[, aconfigurations[[".ID."]], drop=FALSE], test = scenario$testType)
     best_id <- which.min(cranks)[1L]
-    # cand.mean <- colMeans2(results[,aconfigurations$.ID.,drop=FALSE], na.rm=TRUE)
     changes[[step]] <- ab_aux$changed_params
     best_change <- changes[[step]][[best_id]]
     trajectory <- c(trajectory, aconfigurations[[".ID."]][best_id])
-    
-    # Report best
-    # FIXME: This ID does not actually match the configuration ID
-    # The race already reports the best.
+
+    # Report best.
     cat("# Best changed parameters:\n")
     for (i in seq_along(best_change)) {
       cat("#", best_change[i], ":", best_configuration[,best_change[i]], "->",
           aconfigurations[best_id, best_change[i]], "\n")
     }
-  
     best_configuration <- aconfigurations[best_id,,drop=FALSE]
     best_id <- best_configuration[[".ID."]]
     param_names <- param_names[param_names %not_in% best_change]
     step <- step + 1L
   }
-  
-  # Add last configuration and its results
-  # FIXME: This may be overriding the ID of an existing configuration!!!
-  target_configuration[[".ID."]] <- max(all_configurations[[".ID."]]) + 1L
-  all_configurations <- rbind(all_configurations, target_configuration)
-  results <- cbind(results, matrix(lastres, ncol = 1L,
-                                   dimnames=list(seq_nrow(race_state$instances_log),
-                                                 target_configuration[[".ID."]])))
+
   trajectory <- c(trajectory, target_configuration[[".ID."]])
-  
   # Get the overall best
-  cranks <- overall_ranks(results[,trajectory, drop=FALSE], test = scenario$testType)
+  cranks <- overall_ranks(experiments[,trajectory, drop=FALSE], test = scenario$testType)
   best_id <- which.min(cranks)[1L]
-  ## FIXME: At this point, the rownames of all_configurations does not match
-  ## all_configurations[[".ID."]]  That is confusing and a potential source of
-  ## bugs. Instead of fixing it here, we should not generate the discrepancy
-  ## ever.
-  best_configuration <- all_configurations[trajectory[best_id],,drop=FALSE]
-  irace.note("Final best configuration:\n")
+  setDF(all_configurations, rownames = all_configurations[[".ID."]])
+  best_configuration <- all_configurations[trajectory[best_id], , drop=FALSE]
+  irace_note("Final best configuration:\n")
   configurations_print(best_configuration)
 
   # Check for duplicated results:
-  report_duplicated_results(results, all_configurations)
+  report_duplicated_results(experiments, all_configurations)
 
-  # LESLIE: If we use racing we can have a matrix of results that is not
-  # complete, how should we do the plots?
-  # MANUEL: Do not plot anything that was discarded
-  
   save_ablog(complete = TRUE)
 }
 
@@ -532,7 +545,7 @@ ablation_labels <- function(trajectory, configurations)
     current <- configurations[i, , drop = FALSE]
     # Select everything that is NOT NA now and was different or NA before.
     select <- !is.na(current) & (is.na(last) | (current != last))
-    irace.assert(!anyNA(select))
+    irace_assert(!anyNA(select))
     labels[i] <- paste0(param_names[select], "=", current[, select], collapse = "\n")
     last <- current
   }
@@ -575,26 +588,32 @@ plotAblation <- function (ablog, pdf_file = NULL, width = 20,
 {
   type <- trimws(unlist(strsplit(type, ",", fixed=TRUE)))
   type <- match.arg(type, several.ok = TRUE)
-  if (missing(ylab) && ("rank" %in% type)) ylab <- "Rank per instance"
-  
+  if (missing(ylab) && ("rank" %in% type))
+    ylab <- "Rank per instance"
+
   if (missing(ablog) || is.null(ablog)) {
-    irace.error("You must provide an 'ablog' object generated by ablation() or the path to the '.Rdata' file that contains this object.")
+    irace_error("You must provide an 'ablog' object generated by ablation() or the path to the '.Rdata' file that contains this object.")
   }
   ablog <- read_ablogfile(ablog)
   if (!ablog$complete)
     stop("The ablog shows that the ablation procedure did not complete cleanly and only contains partial information")
-  
-  
+
+
   if (!is.null(pdf_file)) {
     if (!is.file.extension(pdf_file, ".pdf"))
       pdf_file <- paste0(pdf_file, ".pdf")
     cat("Creating PDF file '", pdf_file, "'\n", sep="")
     local_cairo_pdf(pdf_file, width = width, height = height, onefile= TRUE)
   }
-  
-  configurations <- ablog$configurations
+
+  configurations <- ablog$allConfigurations
+  # Support irace < 4.2
+  if (is.null(configurations))
+    configurations <- ablog$configurations
+
   trajectory <- ablog$trajectory
-  if (n > 0L) trajectory <- trajectory[seq_len(n+1L)]
+  if (n > 0L)
+    trajectory <- trajectory[seq_len(n+1L)]
   # Generate labels
   labels <- ablation_labels(trajectory, configurations)
   if (!is.null(rename_labels))
@@ -620,16 +639,17 @@ plotAblation <- function (ablog, pdf_file = NULL, width = 20,
       bx <- boxplot(experiments[, trajectory], plot=FALSE)
       if (is.null(ylim)) {
         ylim <- range(bx$stats[is.finite(bx$stats)],
-          bx$out[is.finite(bx$out)], 
+          bx$out[is.finite(bx$out)],
           bx$conf[is.finite(bx$conf)])
       }
     }
-    
-    plot(costs_avg, xaxt = "n", xlab = NA, ylab = ylab, ylim = ylim,
-      type = "b", pch = 19, ...,
+    # Extra whitespace around boxplots.
+    xlim <- c(0.6, length(trajectory) + 0.4)
+    plot(costs_avg, xaxt = "n", xlab = NA, ylab = ylab,
+      xlim = xlim, ylim = ylim, type = "b", pch = 19, ...,
       panel.first = {
         grid(nx = NA, ny = NULL, lwd = 2);
-        abline(h = c(costs_avg[1], tail(costs_avg, n = 1)),
+        abline(h = c(costs_avg[1], utils::tail(costs_avg, n = 1L)),
           col = "lightgray", lty = "dotted", lwd = 2) })
     axis(1, at = seq_along(costs_avg), labels = labels, las = 3)
     if ("boxplot" %in% type) {
@@ -641,9 +661,9 @@ plotAblation <- function (ablog, pdf_file = NULL, width = 20,
 
 #' Read the log file (`log-ablation.Rdata`) produced by [irace::ablation()].
 #'
-#' @param filename Filename that contains the log file saved by [ablation()]. Example: `log-ablation.Rdata`.
-#' 
-#' @return (`list()`)
+#' @param filename `character(1)`\cr Filename that contains the log file saved by [ablation()]. Example: `log-ablation.Rdata`.
+#'
+#' @return `list()`
 #' @concept ablation
 #' @export
 read_ablogfile <- function(filename) read_logfile(filename, name = "ablog")
